@@ -122,16 +122,10 @@ class ConfigurationOption(optparse.Option):
         formatter = getattr(parser, 'cli_formatter')
         if formatter:
             formatter = formatter(dest)
-            setattr(values, dest, formatter(getattr(values, dest)))
+            if formatter:
+                setattr(values, dest, formatter(getattr(values, dest)))
 
 ### plugins for option types
-### TODO: this could use a bit of thought
-### They should probably be classes
-# class Option(object):
-#     def arguments(self, name, value):
-#         """return arguments appropriate for construction of an optparse.Option
-#     def take_action(self, ...):
-#         """do something appropriate based on type"""
 
 class BaseCLI(object):
     """base_cli for all option types"""
@@ -229,7 +223,9 @@ __all__ += [i.__class__.__name__ for i in types.values()]
 class Configuration(optparse.OptionParser):
     """declarative configuration object"""
 
-    options = {} # configuration basis
+    options = {}         # configuration basis definition
+    load_option = 'load' # where to put the load option
+    extend = set()       # if dicts/lists should be extended
 
     def __init__(self, configuration_providers=configuration_providers, types=types, load=None, dump='--dump', **parser_args):
 
@@ -266,7 +262,7 @@ class Configuration(optparse.OptionParser):
         # add option(s) for configuration_providers
         if load:
             self.add_option(load,
-                            dest='load', action='append',
+                            dest=self.load_option, action='append',
                             help="load configuration from a file")
 
         # add an option for dumping
@@ -331,17 +327,24 @@ class Configuration(optparse.OptionParser):
 
     ### methods for adding configuration
 
+    def default_config(self):
+        """configuration defaults"""
+        defaults = {}
+        for key, value in self.items():
+            if 'default' in value:
+                defaults[key] = value['default']
+        return copy.deepcopy(defaults)
+
     def __call__(self, *args):
         """add items to configuration and check it"""
 
+        # start with defaults
+        self.config = self.default_config()
+
+        # add the configuration
         for config in args:
             self.add(config)
 
-        # add defaults if not present
-        for key, value in self.items():
-            if 'default' in value and key not in self.config:
-                self.config[key] = value['default']
-
         # validate total configuration
         self.validate()
         # TODO: configuration should be locked after this is called
@@ -349,28 +352,56 @@ class Configuration(optparse.OptionParser):
     def add(self, config, check=True):
         """update configuration: not undoable"""
 
-        self.check(config) # check config to be added
-        self.config.update(config)
-        # TODO: option to extend; augment lists/dicts
+        # check config to be added
+        self.check(config)
+
+        # add the configuration
+        for key, value in config.items():
+            value = copy.deepcopy(value)
+            if key in self.extend and key in self.config:
+                type1 = type(self.config[key])
+                type2 = type(value)
+                assert type1 == type2 # XXX hack
+                if type1 == dict:
+                    self.config[key].update(value)
+                elif type1 == list:
+                    self.config[key].extend(value)
+                else:
+                    raise NotImplementedError
+            else:
+                self.config[key] = value
 
     ### methods for optparse
     ### XXX could go in a subclass
 
     def cli_formatter(self, option):
-        handler = self.types[self.option_dict[option].get('type')]
-        return getattr(handler, 'take_action', lambda x: 1)
+        if option in self.option_dict:
+            handler = self.types[self.option_dict[option].get('type')]
+            return getattr(handler, 'take_action', lambda x: x)
 
     def option_type(self, name):
+        """get the type of an option named `name`"""
+
         value = self.option_dict[name]
         if 'type' in value:
             return value['type']
         if 'default' in value:
+            default = value['default']
+            if default is None:
+                return None
             return type(value['default'])
 
     def optparse_options(self, parser):
         """add optparse options to a OptionParser instance"""
         for key, value in self.items():
-            handler = self.types[self.option_type(key)]
+            try:
+                handler = self.types[self.option_type(key)]
+            except:
+                # XXX print this for now
+                # ultimately, if an option can't be coerced to a type
+                # we should just not add a CLI handler for it
+                print key, value
+                raise
             args, kw = handler(key, value)
             if not args:
                 # No CLI interface
@@ -387,7 +418,11 @@ class Configuration(optparse.OptionParser):
                            if key in self.option_dict and key in self.parsed])
 
         # deserialize configuration
-        configuration_files = getattr(options, 'load', args)
+        configuration_files = getattr(options, self.load_option, args)
+        if not configuration_files:
+            configuration_files = []
+        if isinstance(configuration_files, basestring):
+            configuration_files = [configuration_files]
         missing = [i for i in configuration_files
                    if not os.path.exists(i)]
         if missing:
@@ -395,7 +430,9 @@ class Configuration(optparse.OptionParser):
         config = []
         for f in configuration_files:
             try:
-                config.append(self.deserialize(f))
+                loaded_config = self.deserialize(f)
+                if loaded_config:
+                    config.append(loaded_config)
             except BaseException, e:
                 parser.error(str(e))
         config.append(cli_config)
